use std::{
    borrow::Cow,
    io::{Cursor, Seek, SeekFrom},
};

use bsky_sdk::{
    api::{
        app::bsky::{
            embed::images::{ImageData, MainData},
            feed::post::{RecordData, RecordEmbedRefs},
        },
        types::string::Datetime,
    },
    BskyAgent,
};

const BSKY_IDENT_ENV_VAR: &str = "BSKY_IDENTIFIER";
const BSKY_PASSWORD_ENV_VAR: &str = "BSKY_PASSWORD";

fn getenv(name: &str) -> String {
    std::env::var(name).expect(&format!("Please specify {} in the env", name))
}

pub(crate) async fn post(
    text: &str,
    img: Option<Box<Cursor<Vec<u8>>>>,
) -> Result<(), Box<dyn std::error::Error>> {
    let agent = BskyAgent::builder().build().await?;
    agent
        .login(getenv(BSKY_IDENT_ENV_VAR), getenv(BSKY_PASSWORD_ENV_VAR))
        .await?;
    let embed = if let Some(mut i) = img {
        i.as_mut().seek(SeekFrom::Start(0))?;
        let out = agent
            .api
            .com
            .atproto
            .repo
            .upload_blob(i.into_inner())
            .await?;
        let embed = RecordEmbedRefs::AppBskyEmbedImagesMain(Box::new(
            MainData {
                images: vec![ImageData {
                    alt: "".into(),
                    aspect_ratio: None,
                    image: out.data.blob,
                }
                .into()],
            }
            .into(),
        ));
        Some(bsky_sdk::api::types::Union::Refs(embed))
    } else {
        None
    };
    agent
        .create_record(RecordData {
            created_at: Datetime::now(),
            embed,
            entities: None,
            facets: None,
            labels: None,
            langs: None,
            reply: None,
            tags: None,
            text: text.into(),
        })
        .await?;
    Ok(())
}
